import board
import digitalio
import busio
import storage
import audiomp3
import audiobusio
import sdcardio
import os
import time
import neopixel

from audiomixer import Mixer

# Boutons de la Maker Pi Pico et LED de PICO
led = digitalio.DigitalInOut(board.LED)
led.direction = digitalio.Direction.OUTPUT
# Eteindre la LED du PICO
led.value = False

# Bouton 1 = GP20 = Musique suivante
button1 = digitalio.DigitalInOut(board.GP20)
button1.direction = digitalio.Direction.INPUT

# Bouton 2 = GP21 = Volime -
button2 = digitalio.DigitalInOut(board.GP21)
button2.direction = digitalio.Direction.INPUT

# Bouton 3 = GP22 = Volime +
button3 = digitalio.DigitalInOut(board.GP22)
button3.direction = digitalio.Direction.INPUT

# Déclaration de la LED NeoPixel
pixel_pin = board.GP28
num_pixels = 1
pixels = neopixel.NeoPixel(pixel_pin, num_pixels, brightness=0.2, auto_write=False)

# Couleurs pour NeoPixel
RED = (255, 0, 0)
YELLOW = (255, 150, 0)
GREEN = (0, 255, 0)
DARK_GREEN = (0, 100, 0)
CYAN = (0, 255, 255)
BLUE = (0, 0, 255)
PURPLE = (180, 0, 255)
WHITE = (50, 50, 50)
BLACK = (0, 0, 0)

# Gestion de la carte SD
spi = busio.SPI(board.GP10, MOSI = board.GP11, MISO = board.GP12)
cs = board.GP15
sd = sdcardio.SDCard(spi, cs)
# Monter la carte SD dans le système de fichiers 
vfs = storage.VfsFat(sd)
storage.mount(vfs, '/sd')

# Liste des fichiers
print (os.listdir("/sd"))

# Garder les fichiers .mp3
mp3files = sorted("/sd/" + filename for filename in os.listdir("/sd")
    if filename.lower().endswith("mp3"))
time.sleep(1)

# Volume de sortie (Réglable de 0 à 10)
# Utliser des valeurs entières pour éviter les erreurs d'approximation
volume = 2

# Liste des fichiers triés
print("La liste contient ", len(mp3files), "fichiers")
print(mp3files)

# Créer instance i2s
i2s = audiobusio.I2SOut(board.GP0, board.GP1, board.GP2)

def play_sound(filename):
    global volume
    # Ouvrir fichier .mp3 et l'envoyer au décodeur
    mp3_filename = f"{filename}"
    mp3 = open(mp3_filename, "rb")
    mp3_decoder = audiomp3.MP3Decoder(mp3)

    # Configurer le mixer
    mixer = Mixer(voice_count=1, sample_rate=mp3_decoder.sample_rate, channel_count=mp3_decoder.channel_count, bits_per_sample=16, samples_signed=True)
    mixer.voice[0].level = volume/10  # Volume initial (Réglable de 0.0 à 1.0)
    # Attacher le mixer à la sortie i2s
    i2s.play(mixer)

    # Jouer le fichier mp3 sur la voie 0 du mixer
    mixer.voice[0].play(mp3_decoder)
    
    while mixer.playing:
        # Arrêter le morceau en cours Bouton 1 = GP20
        # Arrête le morceau en cours et passe au suivant
        if button1.value == False:
            print("Stop", filename, "\n\r")
            mixer.stop_voice()
            mp3.close()

        # Baisser le volume Bouton 2 = GP21
        if button2.value == False:
            if volume > 0:
                volume = volume - 1
                print("Volume - : ", volume, "\n\r")
                mixer.voice[0].level = volume/10
                # Le sleep(0.5) sert de debounce
                time.sleep(0.5)
            else:
                pass
            
        #Augmenter le volume Bouton 3 = GP22    
        if button3.value == False:
            if volume < 10:
                volume = volume + 1
                print("Volume + : ", volume, "\n\r")
                mixer.voice[0].level = volume/10  
                # Le sleep(0.5) sert de debounce
                time.sleep(0.5)
            else:
                pass     

if __name__ == '__main__':
    while True:
        # Lire les fichiers de la liste
        for musique in mp3files:
            # Allumer les LEDs
            led.value = True
            pixels.fill(DARK_GREEN)
            pixels.show()
            print("Playing : ", musique)
            play_sound(musique)
            # Eteindre les LEDs
            led.value = False
            pixels.fill(BLACK)
            pixels.show()
            
            time.sleep(1)
